package gd_access_log

import (
	"context"
	"fmt"
	"gd_gateway/apis"
	"time"

	"github.com/smallnest/rpcx/client"
	"github.com/smallnest/rpcx/protocol"
	"github.com/smallnest/rpcx/share"
)

type GdAccessLogXClient struct {
	xcli client.XClient
}

func (a *GdAccessLogXClient) Init(etcdAddrs []string, basePath, servicePath string) {
	opt := client.Option{
		Retries:        3,
		RPCPath:        share.DefaultRPCPath,
		ConnectTimeout: 10 * time.Second,
		SerializeType:  protocol.JSON,
		CompressType:   protocol.None,
		BackupLatency:  10 * time.Millisecond,
	}

	d := client.NewEtcdDiscovery(basePath, servicePath, etcdAddrs, nil)
	a.xcli = client.NewXClient(servicePath, client.Failtry, client.RandomSelect, d, opt)

}

func (a *GdAccessLogXClient) InitForK8s(servicePath string, k8sServiceName string, k8sServicePort string) {
	opt := client.Option{
		Retries:        1,
		RPCPath:        share.DefaultRPCPath,
		ConnectTimeout: 10 * time.Second,
		SerializeType:  protocol.JSON,
		CompressType:   protocol.None,
		BackupLatency:  10 * time.Millisecond,
	}
	sd := client.NewPeer2PeerDiscovery(fmt.Sprintf("tcp@%s:%s", k8sServiceName, k8sServicePort), "")
	a.xcli = client.NewXClient(servicePath, client.Failtry, client.RandomSelect, sd, opt)
}

func (a *GdAccessLogXClient) LogAddAccessLog(ctx context.Context, req *apis.LogAddAccessLogReq) (reply apis.LogAddAccessLogReply, err error) {
	err = a.xcli.Call(ctx, "LogAddAccessLog", req, &reply)
	return
}

func (a *GdAccessLogXClient) LogAddInvalidAccessLog(ctx context.Context, req *apis.LogAddInvalidAccessLogReq) (reply apis.LogAddInvalidAccessLogReply, err error) {
	err = a.xcli.Call(ctx, "LogAddInvalidAccessLog", req, &reply)
	return
}
